package com.qmth.cqb.genpaper.service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.qmth.cqb.genpaper.context.DetailContext;
import com.qmth.cqb.genpaper.context.PaperContext;
import com.qmth.cqb.genpaper.context.UnitContext;
import com.qmth.cqb.genpaper.model.GenPaperDto;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.dao.PaperStructRepo;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.model.PaperStruct;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.QuesStructType;

@Service
public class GenPaperService {

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperDetailRepo paperDetailRepo;

    @Autowired
    PaperDetailUnitRepo unitRepo;

    @Autowired
    PaperStructRepo paperStructRepo;

    /**
     * 精确组卷，根据设定试卷结构组卷
     * 
     * @param paperIds
     *            取题范围
     * @param paperStruct
     *            设定的试卷结构
     * @param conditions
     *            筛选条件
     * @return
     */
    public Map<String, Object> genPaper(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        LinkedList<Question> questions = new LinkedList<Question>();// 所有试题集合
        PaperStruct paperStruct = paperStructRepo.findOne(genPaperDto.getPaperStructId());
        PaperContext paperContext = new PaperContext(paperStruct, genPaperDto.getConditions());
        List<UnitContext> unitContexts = new LinkedList<UnitContext>();
        List<DetailContext> detailContexts = paperContext.getDetails();
        for (DetailContext dc : detailContexts) {
            unitContexts.addAll(dc.getUnits());
        }
        List<Paper> papers = (List<Paper>) paperRepo.findAll(genPaperDto.getPaperIds());
        for (Paper oldPaper : papers) {
            List<PaperDetailUnit> unitList = unitRepo.findByPaper(oldPaper);
            for (PaperDetailUnit unit : unitList) {
                questions.add(unit.getQuestion());
            }

        }
        Collections.shuffle(questions);
        int maxLoopCount = questions.size() * 2;
        int selectCount = 0;
        int maxCount = unitContexts.size();
        int loopCount = 0;
        while (questions.size() > 0 && (selectCount < maxCount) && loopCount < maxLoopCount) {
            Question question = questions.removeFirst();
            boolean structTypeMatch = false;
            boolean selected = false;
            for (UnitContext uc : unitContexts) {
                if (uc.getUnitStruct().getQuestionType() == question.getQuestionType() && !uc.finish()) {
                    structTypeMatch = true;
                    if (uc.check(question)) {
                        uc.select(question);
                        selected = true;
                        selectCount++;
                        break;
                    }
                }
            }
            if (structTypeMatch && !selected) {
                questions.addLast(question);
            }
            loopCount++;
        }
        Set<Question> quesSet = paperContext.getSelectQues();
        if (quesSet.size() < unitContexts.size()) {
            msg = "匹配的题源不足，请核查";
            return paperMsgMap;
        }
        List<PaperDetail> paperDetails = new ArrayList<PaperDetail>();
        List<PaperDetailUnit> paperDetailunits = new ArrayList<PaperDetailUnit>();
        for (DetailContext dc : detailContexts) {
            List<PaperDetailUnit> unitsTemp = new ArrayList<PaperDetailUnit>();
            List<UnitContext> units = dc.getUnits();
            for (UnitContext uc : units) {
                for (Question ques : quesSet) {
                    if (uc.getUnitStruct().getQuestionType() == ques.getQuestionType()) {
                        PaperDetailUnit pdu = new PaperDetailUnit();
                        pdu.setCreateTime(CommonUtils.getCurDateTime());
                        pdu.setQuestion(ques);
                        pdu.setQuestionType(ques.getQuestionType());
                        pdu.setScore(uc.getUnitStruct().getScore());
                        unitsTemp.add(pdu);
                    }

                }

            }
            paperDetails.add(this.constuctPaperDeatil(unitsTemp, dc.getDetailStruct().getNumber(),
                    dc.getDetailStruct().getName()));
            paperDetailunits.addAll(unitsTemp);
        }
        Paper paper = this.constuctPaperByPaperDetails(paperDetails, genPaperDto);
        this.persistentPaper(paperDetails, paperDetailunits, paper);
        msg = "success";
        paperMsgMap.put("paper", paper);
        paperMsgMap.put("msg", msg);
        return paperMsgMap;
    }

    /**
     * 按分数在选定的试卷中随机抽取题,组成一套新的试卷
     */
    public Map<String, Object> genPaperByScore(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        Map<String, Double> paperMap = genPaperDto.getSimpleParams();
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        if (paperMap != null && paperMap.size() > 0) {
            for (String paperId : paperMap.keySet()) {
                Double totalScore = paperMap.get(paperId);
                Paper oldPaper = paperRepo.findOne(paperId);
                List<PaperDetailUnit> unitList = unitRepo.findByPaper(oldPaper);
                Collections.shuffle(unitList);// 随机乱序之后再根据分数取题
                double sum = 0;
                for (PaperDetailUnit pdu : unitList) {
                    sum += pdu.getScore();
                    if (sum <= totalScore) {
                        selectedUnits.add(pdu);
                    } else {
                        break;
                    }
                }
            }
        }

        Paper paper = this.constuctPaper(selectedUnits, genPaperDto);
        msg = "success";
        paperMsgMap.put("paper", paper);
        paperMsgMap.put("msg", msg);
        return paperMsgMap;

    }

    /**
     * 按每套试卷需要抽取的题目数量随机抽取题,组成一套新的试卷
     * 
     * @return
     */
    public Map<String, Object> genPaperByQuestionNum(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        Map<String, Double> paperMap = genPaperDto.getSimpleParams();
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        if (paperMap != null && paperMap.size() > 0) {
            for (String paperId : paperMap.keySet()) {
                int unitNum = paperMap.get(paperId).intValue();
                Paper oldPaper = paperRepo.findOne(paperId);
                List<PaperDetailUnit> unitList = unitRepo.findByPaper(oldPaper);
                Collections.shuffle(unitList);// 随机乱序之后再取题
                if (unitList.size() >= unitNum) {
                    for (int i = 0; i < unitNum; i++) {
                        selectedUnits.add(unitList.get(i));
                        unitList.remove(i);
                    }
                } else {
                    msg = "题源不足:需要抽取的题目数量要多于试卷中题目数量，请检查";
                    paperMsgMap.put("msg", msg);
                    return paperMsgMap;
                }
                for (PaperDetailUnit pdu : selectedUnits) {
                    if (pdu.getQuestionType().getId() == 6L && pdu.getQuestion().getSubQuestions().size() > 1) {// 假如是套题，要计算下面小题数量
                        selectedUnits.remove(pdu);// 删除掉这道套题
                        for (PaperDetailUnit oldPdu : unitList) {
                            if (pdu.getQuestionType().getId() != 6L) {// 从原来的集合中取一个不是套题的加进去
                                selectedUnits.add(oldPdu);
                                break;
                            }
                        }
                    }
                }

            }
        }
        Paper paper = this.constuctPaper(selectedUnits, genPaperDto);
        msg = "success";
        paperMsgMap.put("paper", paper);
        paperMsgMap.put("msg", msg);
        return paperMsgMap;

    }

    /**
     * 构造大题
     * 
     * @param units
     * @param number
     * @param detailName
     *            大题名称
     * @return
     */
    public PaperDetail constuctPaperDeatil(List<PaperDetailUnit> units, int number, String detailName) {
        PaperDetail pd = new PaperDetail();
        if (units != null && units.size() > 0) {
            Double score = 0d;
            for (PaperDetailUnit pdu : units) {
                score += pdu.getScore();

            }
            pd.setName(detailName);
            pd.setUnitCount(units.size());
            pd.setScore(score);
            pd.setNumber(number);
            pd.setCreateTime(CommonUtils.getCurDateTime());
            for (PaperDetailUnit pdu : units) {
                pdu.setPaperDetail(pd);
            }
        }
        return pd;

    }

    /**
     * 根据大题集合构造paper
     * 
     * @param details
     * @return
     */
    public Paper constuctPaperByPaperDetails(List<PaperDetail> details, GenPaperDto genPaperDto) {
        Paper paper = new Paper();
        if (details != null && details.size() > 0) {
            Double score = 0d;
            for (PaperDetail pd : details) {
                score += pd.getScore();

            }
            paper.setName(genPaperDto.getPaperName());
            paper.setCourseNo(genPaperDto.getCourseNo());
            paper.setCourseName(genPaperDto.getCourseName());
            paper.setCreator(genPaperDto.getCreator());
            paper.setCreateTime(CommonUtils.getCurDateTime());
            paper.setPaperDetailCount(details.size());
            paper.setTotalScore(score);
            for (PaperDetail pdu : details) {
                pdu.setPaper(paper);
            }
        }
        return paper;

    }

    /**
     * 随机构造试卷,根据小题类型 没有定义试卷结构
     * 
     * @param units
     * @param number
     * @param quesName
     * @return
     */
    public Paper constuctPaper(List<PaperDetailUnit> selectedUnits, GenPaperDto genPaperDto) {
        Paper paper = new Paper();
        List<PaperDetailUnit> singUnits = new ArrayList<PaperDetailUnit>();
        List<PaperDetailUnit> multipleUnits = new ArrayList<PaperDetailUnit>();
        List<PaperDetailUnit> boolUnits = new ArrayList<PaperDetailUnit>();
        List<PaperDetailUnit> blankUnits = new ArrayList<PaperDetailUnit>();
        List<PaperDetailUnit> textUnits = new ArrayList<PaperDetailUnit>();
        List<PaperDetailUnit> nestUnits = new ArrayList<PaperDetailUnit>();
        for (PaperDetailUnit pdu : selectedUnits) {
            switch (pdu.getQuestionType()) {
            case SINGLE_ANSWER_QUESTION:
                singUnits.add(pdu);
                break;
            case MULTIPLE_ANSWER_QUESTION:
                multipleUnits.add(pdu);
                break;
            case BOOL_ANSWER_QUESTION:
                boolUnits.add(pdu);
                break;
            case FILL_BLANK_QUESTION:
                blankUnits.add(pdu);
                break;
            case TEXT_ANSWER_QUESTION:
                textUnits.add(pdu);
                break;
            case NESTED_ANSWER_QUESTION:
                nestUnits.add(pdu);
                break;

            }
        }
        int detailNumber = 1;
        PaperDetail singPd = null;
        PaperDetail multipePd = null;
        PaperDetail boolPd = null;
        PaperDetail blankPd = null;
        PaperDetail textPd = null;
        PaperDetail nestPd = null;
        if (singUnits != null && singUnits.size() > 0) {
            singPd = this.constuctPaperDeatil(singUnits, detailNumber, QuesStructType.SINGLE_ANSWER_QUESTION.getName());
            detailNumber++;
        } else if (multipleUnits != null && multipleUnits.size() > 0) {
            multipePd = this.constuctPaperDeatil(multipleUnits, detailNumber,
                    QuesStructType.MULTIPLE_ANSWER_QUESTION.getName());
            detailNumber++;
        } else if (boolUnits != null && boolUnits.size() > 0) {
            boolPd = this.constuctPaperDeatil(boolUnits, detailNumber, QuesStructType.BOOL_ANSWER_QUESTION.getName());
            detailNumber++;
        } else if (blankUnits != null && blankUnits.size() > 0) {
            blankPd = this.constuctPaperDeatil(blankUnits, detailNumber, QuesStructType.FILL_BLANK_QUESTION.getName());
            detailNumber++;
        } else if (textUnits != null && textUnits.size() > 0) {
            textPd = this.constuctPaperDeatil(textUnits, detailNumber, QuesStructType.TEXT_ANSWER_QUESTION.getName());
            detailNumber++;
        } else if (nestUnits != null && nestUnits.size() > 0) {
            nestPd = this.constuctPaperDeatil(nestUnits, detailNumber, QuesStructType.NESTED_ANSWER_QUESTION.getName());
        }
        selectedUnits.clear();
        List<PaperDetail> pds = new ArrayList<PaperDetail>();
        int paperDetailCount = 0;// 大题数量
        Double totalScore = 0d;
        if (singPd != null && singPd.getUnitCount() > 0) {
            paperDetailCount = paperDetailCount + 1;
            totalScore += singPd.getScore();
            pds.add(singPd);
            selectedUnits.addAll(singUnits);
        } else if (multipePd != null && multipePd.getUnitCount() > 0) {
            paperDetailCount = paperDetailCount + 1;
            totalScore += singPd.getScore();
            pds.add(multipePd);
            selectedUnits.addAll(multipleUnits);
        } else if (boolPd != null && boolPd.getUnitCount() > 0) {
            paperDetailCount = paperDetailCount + 1;
            totalScore += singPd.getScore();
            pds.add(boolPd);
            selectedUnits.addAll(boolUnits);
        } else if (blankPd != null && blankPd.getUnitCount() > 0) {
            paperDetailCount = paperDetailCount + 1;
            totalScore += singPd.getScore();
            pds.add(blankPd);
            selectedUnits.addAll(blankUnits);
        } else if (textPd != null && textPd.getUnitCount() > 0) {
            paperDetailCount = paperDetailCount + 1;
            totalScore += singPd.getScore();
            pds.add(textPd);
            selectedUnits.addAll(textUnits);
        } else if (nestPd != null && nestPd.getUnitCount() > 0) {
            paperDetailCount = nestPd.getUnitCount() + paperDetailCount;
            totalScore += singPd.getScore();
            pds.add(nestPd);
            selectedUnits.addAll(nestUnits);
        }
        paper.setName(genPaperDto.getPaperName());
        paper.setCourseName(genPaperDto.getCourseName());
        paper.setCourseNo(genPaperDto.getCourseNo());
        paper.setCreator(genPaperDto.getCreator());
        paper.setCreateTime(CommonUtils.getCurDateTime());
        paper.setTotalScore(totalScore);
        paper.setPaperDetailCount(paperDetailCount);
        // 数据入库
        paper = this.persistentPaper(pds, selectedUnits, paper);
        return paper;

    }

    /**
     * 数据入库
     * 
     * @param details
     * @param selectedUnits
     * @param paper
     * @return
     */
    public Paper persistentPaper(List<PaperDetail> details, List<PaperDetailUnit> selectedUnits, Paper paper) {
        for (PaperDetail pd : details) {
            pd.setPaper(paper);
        }
        for (PaperDetailUnit pdu : selectedUnits) {
            pdu.setPaper(paper);
        }
        unitRepo.save(selectedUnits);
        paperDetailRepo.save(details);
        paperRepo.save(paper);
        return paper;

    }

}
