package com.qmth.cqb.paper.service;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.qmth.cqb.paper.model.PaperSearchInfo;
import com.qmth.cqb.utils.BeanCopierUtil;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.PaperStatus;
import com.qmth.cqb.utils.enums.PaperType;
import org.apache.commons.collections4.IteratorUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import com.qmth.cqb.paper.dao.ExamPaperRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.model.ExamPaper;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;

/**
 * Created by songyue on 16/12/28.
 */
@Service
public class PaperService {
	
	@Autowired
	PaperRepo paperRepo;
	
	@Autowired
	PaperDetailService paperDetailService;
	
	@Autowired
	ExamPaperRepo examPaperRepo;

	/**
	 * 查询所有已导入试卷
	 * @param paperSearchInfo
	 * @param curPage
	 * @param pageSize
	 * @return
	 */
	public Page<Paper> getImportPapers(PaperSearchInfo paperSearchInfo,
									   int curPage,
									   int pageSize){
		formatPaperSearchInfo(paperSearchInfo);
		Paper importPaper = BeanCopierUtil.copyProperties(paperSearchInfo,Paper.class);
		importPaper.setPaperType(PaperType.IMPORT);
		return paperRepo.findAll(Example.of(importPaper),new PageRequest(curPage - 1,pageSize));
	}

	/**
	 * 查询所有已组试卷
	 * @param paperSearchInfo
	 * @param curPage
	 * @param pageSize
	 * @return
	 */
	public Page<Paper> getGenPapers(PaperSearchInfo paperSearchInfo,
									int curPage,
									int pageSize){
		formatPaperSearchInfo(paperSearchInfo);
		Paper importPaper = BeanCopierUtil.copyProperties(paperSearchInfo,Paper.class);
		importPaper.setPaperType(PaperType.GENERATE);
		return paperRepo.findAll(Example.of(importPaper),new PageRequest(curPage - 1,pageSize));
	}
	
	/**
	 * 查询考试试卷
	 * @param id
	 * @param courseCode
	 * @param groupCode
	 * @return
	 */
    public List<Paper> listExamPapers(long id,String courseCode,String groupCode){
    	List<Paper> papers = new ArrayList<Paper>();
     	ExamPaper examPaper = new ExamPaper();
    	examPaper.setExamId(id);
    	examPaper.setCourseCode(courseCode);
    	examPaper.setGroupCode(groupCode);
    	Example<ExamPaper> example = Example.of(examPaper); 
    	List<ExamPaper> examPapers = examPaperRepo.findAll(example);
    	for(ExamPaper ePaper:examPapers){
    		Paper paper = paperRepo.findOne(ePaper.getPaperId());
    		papers.add(paper);
    	}
		return papers;
    }
    
    
    /**
     * 设置考试试卷
     * @param examId
     * @param courseCode
     * @param groupCode
     * @param paperId
     * @return
     */
    public void joinToExamPaper(long examId,
								String courseCode,
								String groupCode,
								String paperId){
		ExamPaper examPaper = new ExamPaper();
		examPaper.setExamId(examId);
		examPaper.setGroupCode(groupCode);
		examPaper.setCourseCode(courseCode);
		examPaper.setPaperId(paperId);
		examPaperRepo.save(examPaper);
    }
    
    public void releaseExamPaper(long examId,
									String courseCode,
									String groupCode,
									String paperId){
		ExamPaper examPaper = new ExamPaper();
		examPaper.setExamId(examId);
		examPaper.setGroupCode(groupCode);
		examPaper.setCourseCode(courseCode);
		examPaper.setPaperId(paperId);
		examPaperRepo.delete(examPaper);
    }
    
    
    public Set<String> listGroupCodes(long examId,String courseCode){
    	Set<String> groupSet = new HashSet<String>();
    	ExamPaper examPaper = new ExamPaper();
    	examPaper.setExamId(examId);
		examPaper.setCourseCode(courseCode);
		List<ExamPaper> examPapers = examPaperRepo.findAll(Example.of(examPaper));
		for(ExamPaper expaper:examPapers){
			groupSet.add(expaper.getGroupCode());
		}
		return groupSet;
    }
    
    
    public void deletGroupCode(long examId,String courseCode,String groupCode){
    	ExamPaper examPaper = new ExamPaper();
    	examPaper.setExamId(examId);
		examPaper.setCourseCode(courseCode);
		examPaper.setCourseCode(courseCode);
		examPaperRepo.delete(examPaper);
    }
    
    
    /**
     * 根据试卷ID获取试卷下面的大题
     * @param id
     * @return
     */
    public List<PaperDetail> findPaperDetailsById(String id){
        return paperDetailService.getPaperDetailsByPaper(paperRepo.findOne(id));
    }

	/**
	 * 批量删除试卷
	 * @param paperIds
     */
	public void deletePapers(List<String> paperIds){
		List<Paper> papers = CommonUtils.toList(paperRepo.findAll(paperIds));
		paperDetailService.deletePaperDetailsByPapers(papers);
		paperRepo.delete(papers);
	}

	/**
	 * 批量通过试卷
	 * @param paperIds
	 */
	public void passPapers(List<String> paperIds){
		List<Paper> papers = CommonUtils.toList(paperRepo.findAll(paperIds));
		papers.stream().forEach(paper -> {paper.setPaperStatus(PaperStatus.PASS);});
		paperRepo.save(papers);
	}

	/**
	 * 批量不通过试卷
	 * @param paperIds
	 */
	public void noPassPapers(List<String> paperIds){
		List<Paper> papers = CommonUtils.toList(paperRepo.findAll(paperIds));
		papers.stream().forEach(paper -> {paper.setPaperStatus(PaperStatus.NOPASS);});
		paperRepo.save(papers);
	}

	/**
	 * 批量待审核试卷
	 * @param paperIds
	 */
	public void initPapers(List<String> paperIds){
		List<Paper> papers = CommonUtils.toList(paperRepo.findAll(paperIds));
		papers.stream().forEach(paper -> {paper.setPaperStatus(PaperStatus.DRAFT);});
		paperRepo.save(papers);
	}

	public void formatPaperSearchInfo(PaperSearchInfo paperSearchInfo){
		if(StringUtils.isEmpty(paperSearchInfo.getCourseNo())){
			paperSearchInfo.setCourseNo(null);
		}
		if(StringUtils.isEmpty(paperSearchInfo.getCreateTime())){
			paperSearchInfo.setCreateTime(null);
		}
		if(StringUtils.isEmpty(paperSearchInfo.getCreator())){
			paperSearchInfo.setCreator(null);
		}
		if(StringUtils.isEmpty(paperSearchInfo.getName())){
			paperSearchInfo.setName(null);
		}
	}

}
